# Generated by Django 2.2 on 2021-05-20 17:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('statusboard', '0022_auto_20201204_0856'),
    ]

    operations = [
        migrations.CreateModel(
            name='Recipient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254, unique=True, verbose_name='email')),
                ('services', models.ManyToManyField(to='statusboard.Service', verbose_name='services')),
            ],
            options={
                'verbose_name': 'recipient',
                'verbose_name_plural': 'recipients',
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('from_status', models.IntegerField(blank=True, choices=[(0, 'Operational'), (1, 'Performance issues'), (2, 'Partial outage'), (3, 'Major outage')], null=True)),
                ('to_status', models.IntegerField(choices=[(0, 'Operational'), (1, 'Performance issues'), (2, 'Partial outage'), (3, 'Major outage')])),
                ('service', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='statusboard.Service')),
            ],
            options={
                'verbose_name': 'notification',
                'verbose_name_plural': 'notifications',
            },
        ),
    ]



# Copyright (C) 2021 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from django.core.management.base import BaseCommand

from statusboard_notify.models import Notification
from statusboard_notify.utils import send_notification_mail
from statusboard_notify.utils import send_notification_telegram


class Command(BaseCommand):
    help = "Send statusbord notifications"

    def add_arguments(self, parser):
        parser.add_argument('-n', '--dry-run', action="store_true")

    def handle(self, *args, **kwargs):
        notifications = Notification.objects.all()
        if not notifications.exists():
            self.stdout.write("Notification queue is empty")
            return

        for dest, func in (
            ("telegram", send_notification_telegram),
            ("email", send_notification_mail),
        ):
            self.stdout.write("Sending {} notifications via {}".format(
                notifications.count(),
                dest,
            ))
            if not kwargs["dry_run"]:
                try:
                    func(notifications)
                except Exception as e:
                    self.stderr.write(e)

        self.stdout.write("Removing {} notifications from the queue".format(
            notifications.count()
        ))
        if not kwargs["dry_run"]:
            try:
                notifications.delete()
            except Exception as e:
                self.stderr.write(e)



# Copyright (C) 2021 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from django.db import models
from django.dispatch import receiver
from django.db.models.signals import post_save
from django.utils.translation import gettext_lazy as _

from statusboard.models import Service, SERVICE_STATUSES


class Recipient(models.Model):
    """Recipient for the notifications. Each recipient can receive
    notifications for a subset of services."""
    email = models.EmailField(unique=True, verbose_name="email")
    services = models.ManyToManyField(Service, verbose_name=_("services"))

    def notifications(self, qs=None):
        if qs is None:
            qs = Notification.objects.all()

        return qs.filter(service__in=self.services.all())

    class Meta:
        verbose_name = _('recipient')
        verbose_name_plural = _('recipients')


class Notification(models.Model):
    """Pending notifications."""
    service = models.OneToOneField(Service, on_delete=models.deletion.CASCADE)
    from_status = models.IntegerField(choices=SERVICE_STATUSES, null=True, blank=True)
    to_status = models.IntegerField(choices=SERVICE_STATUSES)

    class Meta:
        verbose_name = _('notification')
        verbose_name_plural = _('notifications')


@receiver(post_save, sender=Service)
def update_notification(sender, instance, **kwargs):
    """Update the state of a pending notification. A status change from
    "operational" to "performance issues" (or viceversa) is ignored.
    The notification is removed when a service status changes to operational or
    "performance issues". The pending notification keeps track of the first and
    last status: if the service status changes again before the notification is
    sent, this function overwrites the final status.
    """
    if any([
        instance._status == instance.status,
        (instance._status, instance.status) in ((0, 1), (1, 0))
    ]):
        # Ignore if the status is unchanged or the change is between 0
        # (operational) and 1 (performance issues).
        return
    else:
        try:
            n = Notification.objects.get(service=instance)
            if n.from_status == instance.status:
                # If the notification exists and the new final status is equals
                # to the starting status of the notification, remove it.
                n.delete()
            else:
                # Else, update the final status.
                n.to_status = instance.status
                n.save()
        except Notification.DoesNotExist:
            # Create the notification if it doesn't exist.
            n = Notification(service=instance, from_status=instance._status,
                             to_status=instance.status)
            n.save()



# Copyright (C) 2021 Emanuele Di Giacomo <emanuele@digiacomo.cc>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from django.conf import settings
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

import pypandoc

try:
    telegram = None
    import telegram
except ImportError:
    pass


from .models import Recipient


def send_notification_mail_for_recipients(recipients, notifications):
    # Se non ci sono notifiche, non invio la mail
    if not notifications.exists():
        return

    try:
        sender = settings.STATUSBOARD_NOTIFY_EMAIL_SENDER
    except AttributeError:
        return

    html_msg, plain_msg = render_notification_message(notifications)

    try:
        subject = settings.STATUSBOARD_NOTIFY_EMAIL_SUBJECT
    except:
        subject = _("[statusboard] Service status updates")

    for recipient in recipients:
        send_mail(subject, plain_msg, sender, [recipient],
                  fail_silently=False, html_message=html_msg)


def send_notification_mail(notifications):
    try:
        send_notification_mail_for_recipients(
            settings.STATUSBOARD_NOTIFY_EMAIL_RECIPIENTS,
            notifications,
        )
    except AttributeError:
        pass

    for recipient in Recipient.objects.all():
        send_notification_mail_for_recipients(
            [recipient.email],
            recipient.notifications(notifications),
        )


def render_notification_message(notifications):
    html_msg = render_to_string('statusboard_notify/email.html', {
        'notifications': notifications,
    })
    plain_msg = pypandoc.convert_text(html_msg, "markdown_strict",
                                      format="html")
    return html_msg, plain_msg


def send_notification_telegram(notifications):
    if not telegram:
        pass

    try:
        token = settings.STATUSBOARD_NOTIFY_TELEGRAM_TOKEN
        chat_id = settings.STATUSBOARD_NOTIFY_TELEGRAM_CHAT_ID
    except AttributeError:
        return

    bot = telegram.Bot(token=token)
    for notification in notifications:
        bot.send_message(chat_id=chat_id,
                         text=render_notification_telegram(notification),
                         parse_mode=telegram.ParseMode.MARKDOWN)


def render_notification_telegram(notification):
    status_emoji = {
        "0": "\U0001F7E2",  # 🟢
        "1": "\U0001F535",  # 🔵
        "2": "\U0001F7E0",  # 🟠
        "3": "\U0001F534",  # 🔴
    }
    if notification.from_status is not None:
        end_msg = _(
            "changed from __%(fromst)s__ to __%(tost)s__"
        ) % {
            'fromst': notification.get_from_status_display(),
            'tost': notification.get_to_status_display(),
        }
    else:
        end_msg = _(
            "created with status __%(tost)s__"
        ) % {
            'tost': notification.get_to_status_display(),
        }

    msg = "{emoji} **{name}** {end_msg}".format(
        emoji=status_emoji.get(str(notification.to_status)),
        name=notification.service.name,
        end_msg=end_msg
    )

    return msg



import os
import re

from setuptools import find_packages, setup


def get_version(package):
    # Thanks to Tom Christie
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


version = get_version('statusboard_notify')

setup(
    name="django-statusboard-notify",
    version=version,
    packages=find_packages(include=["statusboard_notify", "statusboard_notify.*"]),
    include_package_data=True,
    license='GPLv2+',
    description='Utility for django-statusboard to notify status changes',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url='http://github.com/edigiacomo/django-statusboard-notify',
    author='Emanuele Di Giacomo',
    author_email="emanuele@digiacomo.cc",
    python_requires=">=3.6",
    install_requires=[
        'django>=2.2', 'django-statusboard', 'pypandoc',
    ],
    extras_require={
        'with_telegram': 'python-telegram-bot',
        'with_matrix': 'matrix-nio',
    },
    test_suite="runtests.runtests",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
